
' ZBasic 5.0 and FutureBASIC CRC32 calculation routine
' 68x0 Assembler - Michael Burton, 1992
'                  Michael Lininger, 1992


GOTO "Main_Start"


LONG FN UpdateCRC32(CRCVal&,CRCTbl&,Buffer&,Bytes%)
  
' Last Modified (6/13/92)
' Long Function ( 6/92  Michael Burton   )
'               ( 6/92  Michael Lininger )
  
' CRCVal& is the current value of the 32-bit CRC
' CRCTbl& is the address of the table of CRC values   (STRIPADDRESS for 32bit clean)
' Buffer& is the address of the current block of data
' Bytes% is the number of bytes in the current block  (max. block size is 32k)
  
' We could have used INLINE Assembler Code, but for Z5 folks we will stick with MACHLG
  		
  		LONG IF Bytes% > 0
    			REGISTER(D2) = CRCVal&
    			REGISTER(A0) = CRCTbl&
    			REGISTER(A1) = Buffer&
    			REGISTER(D3) = Bytes%
    			MACHLG &5343           :                   '      SUBQ    #1,D3            ;fix for DBRA
    			MACHLG &4280           :                   '@loop:CLR.L      D0
    			MACHLG &4281           :                   '      CLR.L      D1
    			MACHLG &1002           :                   '      MOVE.B     D2,D0         ;use low byte of CRCVal
    			MACHLG &1219           :                   '      MOVE.B     (A1)+,D1	     ; and data byte
    			MACHLG &B340           :                   '      EOR        D1,D0         ; to create index into
    			MACHLG &E540           :                   '      ASL     #2,D0            ; 32-bit CRC table
    			MACHLG &2230, &0000    :                   '      MOVE.L     0(A0,D0.W),D1
    			MACHLG &4202           :                   '      CLR.B      D2            ;clear low byte
    			MACHLG &E09A           :                   '      ROR.L   #8,D2            ;shift right 8 bits
    			MACHLG &B382           :                   '      EOR.L      D1,D2         ;tbl value XOR CRCVal
    			MACHLG &51CB, &FFE8    :                   '      DBRA       D3,@loop      ;loop 'til done
    			CRCVal& = REGISTER(D2)
  		END IF

END FN = CRCVal&




"Main_Start"

' (6/92  Michael Lininger )

DEFSTR LONG                                       :' CRC32 is 4 bytes long

CRCHndl& = FN GETRESOURCE(CVI("CRC#"),128)        :' Get Handle to CRC32 Lookup Table
LONG IF CRCHndl& = 0                              :' If Z5, Non Project FB or CRC32 Table not found
  		CRCFile$ = "COUNTERPoint:CRC_32 Data Table":' Where is the CRC32 Resource located
  		ResRef% = FN OPENRESFILE(CRCFile$)          :' Open the Resource with the CRC32 Table
  		CRCHndl& = FN GETRESOURCE(CVI("CRC#"),128)  :' Get Handle to CRC32 Lookup Table
END IF
OSErr% = FN RESERROR                              :' Clear any Resource Errors, Should be 0

CRCTbl& = FN STRIPADDRESS(USR 3(CRCHndl&))        :' Locks CRCHndl& Block down and returns 
                                                  :' Pointer (Address) 32 bit clean

' This demo code shows how to calculate a CRC_32 value for any file
' with a Data Fork.

DO
   fileName$ = FILES$(1,"",,Vol%)                 :' Get file to process
   LONG IF fileName$ > ""
    OPEN "R",1,fileName$,0,Vol%
    workSize& = 0
    newCRC& = 0
    CRCVal& = &HFFFFFFFF                          :' Initialize CRCVal& to -1
    Length& = LOF(1,1)                            :' Get length of the file to work on
    
    LONG IF Length& > 32000                       :' Do we need to process multiple blocks
       Bytes% = 32000
    XELSE
      Bytes% = LOF(1,1)
    END IF
    
     DO
      CLEAR Bytes%,4                              :' Clear (set aside) Space _newHandle
      RECORD #1,0,workSize&                       :' Position pointer, in case multiple reads
      READ FILE #1,MEM(4+40),Bytes%               :' Block Read file section into _newHandle

      newCRC& = FN UpdateCRC32(CRCVal&,CRCTbl&,MEM(4+40),Bytes%):' Call the CRC32 function

      workSize& = workSize& + Bytes%              :' Update file pointer
       LONG IF workSize& + Bytes% > Length&
        Bytes% = Length& - workSize&              :' Bytes left to process
      END IF
    UNTIL Bytes% = 0                              :' No more bytes in file to process
    CLOSE #1             
    newCRC& = newCRC& XOR &HFFFFFFFF              :' eXclusive OR newCRC& with -1
    CLEAR 0,4                                     :' Free up space _newHandle
    
    CLS                                           :' Print the Results
    PRINT "File Length:";Length&;"Bytes"
    PRINT "  File Info: ";fileName$;"   crc_32 Value: ";HEX$(newCRC&)
    
  END IF
UNTIL fileName$ = ""

CRCTbl& = USR 6(CRCHndl&)                         :' Dispose of Memory Block used for CRC Table
CALL RELEASERESOURCE(CRCHndl&)                    :' Release Resource handle to CRC Table
                                                  :' Free up memory

IF ResRef% <> 0 THEN CALL CLOSERESFILE(ResRef%)   :' If in Z5, Close External Resource File

END


' --------------------------------------------------

' This demo code shows how to use CRC_32 with String$ variables........

' CRCVal& = &HFFFFFFFF                            :' Initialize CRCVal& to -1

' Block$  = "Michael Lininger"                    :' Data block to determine CRC32 for              
' Buffer& = VARPTR(Block$)+1                      :' Get Address of Data Block (+1 since we are
                                                  :' using a string$ and don't want to include the
                                                  :' the length bit in the CRC32 calculation
' Bytes%  = LEN(Block$)                           :' How long is the data block (Max. 32k)
                                                  :' or Bytes% = PEEK WORD(Buffer&-1)

' newCRC& = FN UpdateCRC32(CRCVal&,CRCTbl&,Buffer&,Bytes%):' Call the CRC32 Function

' newCRC& = newCRC& XOR &HFFFFFFFF                :' eXclusive OR newCRC& with -1


' PRINT "                  Resource Ref  ";ResRef%
' PRINT " Pointer to CRC Table Resource  ";CRCTbl&
' PRINT "             Actual Data Block   ";Block$
' PRINT "Calcuated CRC value for Block$  ";newCRC&
' PRINT "    (HEX) CRC value for Block$   ";HEX$(newCRC&)
' PRINT "           (HEX) CRC should be   76F1051E"
'                                         For   Mike Lininger   data Block


' A couple of other CRC32 values for testing...
' CRC Hex for 'Michael Burton' = D26E97E7
' CRC Hex for 'Michael Lininger' = 76F1051E
' CRC Hex for 'M' = DA6FD2A0
